/*****************************************************************************
 * File: sak2_serial.h
 * Author: Jonathan Gips and Mike Sung, MIT Wearable Computing Group
 * Revised: Ryan Aylward
 * http://www.media.mit.edu/wearables/
 * Adapted from hackfest_serial, hoarder_serial
 * Copyright (C) 2003 MIT Media Lab
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/
#ifndef SAK2SERIAL_H
#define SAK2SERIAL_H

#include <stdio.h>
#include <termios.h>
#include <fcntl.h>
#include <sys/types.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>

#define SAK_DEBUG 0

#define MODEMDEVICE "/dev/ttyS0"
//#define MODEMDEVICE "/dev/ttyS1"  

#define BAUDRATE B115200
#define _POSIX_SOURCE 1 /* POSIX compliant source */
#define FALSE 0
#define TRUE 1

#define SAK2V2_BASE_SAMP_CLOCK 0

typedef enum {SAK2V2_ACCEL_0 = 0, SAK2V2_ACCEL_1, SAK2V2_TAG_READER,
	      SAK2V2_POLAR, SAK2V2_EKG, SAK2V2_GSR, SAK2V2_TEMP, 
	      SAK2V2_NUM_DEVICES} sak2v2_devName;

typedef enum {SAK2V2_ERROR_NONE} sak2v2_error;

/* open serial port */

int sak2v2_init_serial();

/* close serial port */

int sak2v2_close_serial();

int sak2v2_set_device(sak2v2_devName devEnum, int enable);

int sak2v2_configure_devices();

int sak2v2_verify_device_configurations();

int sak2v2_read_devices();

int sak2v2_get_samples(sak2v2_devName dev, char **buf_ptr, int *bps, int *num_samp, struct timeval *time);

int sak2v2_start_streaming();

int sak2v2_stop_streaming();

int sak2v2_clear_configuration();

int sak2v2_echo(int size);

#endif








