/* 
 * LinuxAudioSignal.c, by Rich DeVaul.  Copyright (C) 2004, MIT Media Lab.
 *
 * Generates integer audio signals of specified rate and dimension
 * (stereo, mono) using the linux audio device and the Enchantment
 * signal system.
 * 
 */


#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>

#include <math.h>

#include "linux_audio.h"

#include <enchantment/signals.h>
#include <enchantment/types.h>

/**

 * Generates an integer-valued audio signal of the requested sampling
 * rate and precision.  Currently only mono signals are supported.

 */

int main(int argc, char *argv[]) 
{
  char *outputUNL;
  char *device;

  Signal *out;

  signed short *buffer;
  char *output;
  int rate, channels, vlength, rrate, rchannels;
  int size;
  int i;
  int length;
  struct timeval outTimestamp;
 
  if (argc < 4) 
    {
      printf("Usage: LinuxAudioSignal {output unl} {rate} {channels} [size] [vector length] [device]\n");
      exit(0);
    }

  outputUNL = argv[1];
  rate=atoi(argv[2]);
  channels=atoi(argv[3]);
  size = (argc > 4) ? size=atoi(argv[4]) : 2;
  vlength = (argc > 5) ? vlength=atoi(argv[5]) : DEFAULT_AUDIO_BUFFER_SIZE;
  device = (argc > 6) ? argv[6] : DEFAULT_INPUT_AUDIO_DEVICE;

  switch(channels) {
  case 1:
    break;
  case 2:
    fprintf(stderr,"sorry, stereo not yet supported");
    exit(EXIT_FAILURE);
  default:
    fprintf(stderr,"bad number of channels: %d\n",channels);
    exit(EXIT_FAILURE);
  }

  rrate = rate;
  rchannels = channels;
				/* configure audio */

  if (! linux_init_read_sound(device,	/* default device */
			      &rate,	/* sampling rate */
			      &channels,	/* only mono supported */
			      0)) {	/* default format */
    fprintf(stderr,"%s: Unable to configure linux audio input\n",
	    argv[0]);
    exit(EXIT_FAILURE);
  }

  if (rrate != rate)
    printf("Requested %d Hz; using %d Hz instead\n", rrate, rate);
  if (rchannels != channels)
    printf("Reuqested %d channels; using %d instead\n", rchannels, channels);
  
  buffer= (signed short *)(malloc(sizeof(signed short)*vlength));
  output= (char *)(malloc(size*vlength));

  out = NewSignalGenUNL(outputUNL);
  if (!out)
    {
      printf("Error allocating signalGen\n");
      exit(1);
    }
  
  SignalSetType(out,  T_INT16);
  SignalSetDimension(out, 1, 1, "Mono Audio",NULL);
  SignalSetSampleRate(out, rate,vlength);

  while (1) 
    {
      SignalGenAccept(out);
      length= linux_audio_read(buffer,vlength);

      if (length < vlength) {
	fprintf(stderr,"%s: warning: short audio read: %d\n",
		argv[0],length);
      }

      for (i=0; i < vlength; i++) {
#ifdef VERBOSE_DEBUG
	printf("%d%c",buffer[i], (i%10==0 || i==vlength-1) ? '\n' : ' ' ); 
#endif
	switch(size) {
	case 1:
	  ((char *)output)[i] = ( char )buffer[i];
	  break;
	case 2:
	  ((signed short *)output)[i] = buffer[i];
	  break;
	case 4:
	  ((int *)output)[i] = (int) buffer[i];
	  break;
	}
      }
      SignalGenAccept(out);
      gettimeofday(&outTimestamp,NULL);
      SignalSend(out, &outTimestamp, (char *)output);
    }

  return 0;
}


