/*****************************************************************************
*
* Author: Jonathan Gips, MIT Wearable Computing Group
* http://www.media.mit.edu/wearables/
* Copyright (C) 2005 MIT Media Lab
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
*****************************************************************************/

#ifndef INFLUENCECHAIN_HPP
#define INFLUENCECHAIN_HPP

#include "XMLObject.hpp"
#include "Gaussian.hpp"


class Chain : public XMLObject {
public:
  typedef enum ObservationType {NONE, DISCRETE, CONTINUOUS} ObservationType;

protected:
  void clear();
  
  char *name; // Label for this chain
  char *obsUNL; //UNL where observations coming from
  char *stateUNL; //UNL where state inferences are going
  
  int numStates; //Number of unobserved states
  Vector *priors; //Prior probabilities for states

  
  ObservationType obsType;
  
  int dimObs; //Dimensionality of observations
  Matrix *B_disc; //B Conditional Probability Table for Discrete Observations
  Gaussian *B_cont; //
  

  //----For xml parsing
  XMLVector *priorsXML; 
  XMLMatrix *B_discXML;

  typedef enum XMLState {CHAIN, NAME, OBSUNL, STATEUNL, NUMSTATES, PRIORS, 
			 OBSTYPE, DIMOBS, B_DISC, B_CONT, SKIP} XMLState;
  
  XMLState xml_state;
  int bLoadIdx;  //Current index of gaussian being loaded for B values

public:
  const char *getName() const { return name; }
  const char *getObservationUNL() const {return obsUNL; }
  const char *getStateUNL() const { return stateUNL; }
  bool setObservationUNL(const char *s);
  bool setStateUNL(const char *s);
  int getNumStates() const { return numStates; }
  Vector *getPriors() { return priors; }
  ObservationType getObservationType() const { return obsType; }
  int getDimObservations() const { return dimObs; }
  Matrix *getDiscreteB() { return B_disc; }
  Gaussian *getContinuousB(int stateIdx) { return (B_cont == NULL) ? NULL : &B_cont[stateIdx]; }
  Chain();
  virtual ~Chain();
  
  virtual void show();
  
  virtual void startXMLElement(const char *name, const char **atts);
  virtual void processXMLCharacterData(const char *s, int len);
  virtual void endXMLElement(const char *name);
};


#endif

