/*****************************************************************************
*
* Authors: Jonathan Gips, Mark Blum, Wen Dong, MIT Wearable Computing Group
* http://www.media.mit.edu/wearables/
* Copyright (C) 2005 MIT Media Lab
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
*****************************************************************************/

#ifndef INFLUENCEMODEL_HPP
#define INFLUENCEMODEL_HPP

#include "XMLObject.hpp"
#include "Gaussian.hpp"
#include "Chain.hpp"

extern "C" {
#include "hd_influence.h"
}

#define MAX_VERSION 1.0

class Classification;


class InfluenceModel : public XMLObject {
protected:
  Matrix **A; //array of pointers to matrices that is numChains x numChains
  Matrix *D; //single matrix

  float version;
  char *versionXML;
  int numChains;
  Chain *chains;
  
  //--- Used for loading only
  XMLMatrix *AiXML; //an A sub matrix used for loading
  XMLMatrix *DXML; //alpha matrix
  int c1XML;
  int c2XML;
  XMLMatrix *HXML; //for added complexity...not used
  
  typedef enum XMLState {INFLUENCEMODEL, VERSION, NUMCHAINS, 
			 CHAIN, A_MATRIX, A_SUB, C1, C2, TRANS_MAT,
			 D_MATRIX, H_MATRIX, SKIP} XMLState;
  
  XMLState xml_state;
  int chainLoadIdx;
 
  virtual void startXMLElement(const char *name, const char **atts);
  virtual void processXMLCharacterData(const char *s, int len);
  virtual void endXMLElement(const char *name);
  
  void clear();

protected:
  static int DEFAULT_HD_T;
  static int HD_VIEW_MULT;
  int hdT;  //sample window size
  hd_sample_view *hdSampView;
  hd_influence *hdInf; //the influence model
  int hdSampT;

  virtual void resetSample(hd_sample *hdSamp);
  virtual int initModel();
  virtual void freeModel();
public:
  //NaN, set in constructor
  double NaN;

  int getNumChains() { return numChains; }
  bool setObservationUNL(int c, const char *s);
  bool setStateUNL(int c, const char *s);
  const Chain *getChain(int c);
  Matrix *getD() { return D; }
  bool setD(Matrix *d);
  Matrix *getA(int c1, int c2);
  bool setA(Matrix *m, int c1, int c2);

  InfluenceModel(int hdT = DEFAULT_HD_T);
  virtual ~InfluenceModel();

  virtual int loadXML(char *file);
  virtual void classify(Classification *classification, double **observations);
  virtual void show();
};

#endif

