/*****************************************************************************
*
* Author: Steve Dunn, MIT Wearable Computing Group
* http://www.media.mit.edu/wearables/
* Copyright (C) 2002 MIT Media Lab
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
*****************************************************************************/

#ifndef MIXTURE_MODEL_HPP
#define MIXTURE_MODEL_HPP

#include "XMLObject.hpp"
#include "Gaussian.hpp"

#define MAX_VERSION 1.0

class Classification;

class MixtureModel : public XMLObject {
protected:
  double *readRealArray(char *buf, int dim);  
  void clear();
  
  float version;
  char *versionXML;
  double scalar;
  Gaussian *gaussians;
  XMLVector *priors;
  int numGaussians;
  int dimension;
  
  typedef enum XMLState {MIXTUREMODEL, VERSION, DIMENSION, 
			 SCALAR, NUMCLASSES, PRIORS, GAUSSIAN, SKIP} XMLState;
  XMLState xml_state;
  int gaussianLoadIdx;
  
  virtual void startXMLElement(const char *name, const char **atts);
  virtual void processXMLCharacterData(const char *s, int len);
  virtual void endXMLElement(const char *name);
  
  
public:
  
  double getScalar() { return scalar; }
  int getCount() { return numGaussians; }
  int getDim() { return dimension; }
  Vector *getPriors() { return priors->v; }
  Gaussian *getGaussian(int i) 
  { 
    if((gaussians == NULL) || (i >= numGaussians))
      return NULL;
    return &(gaussians[i]);
  }
  
  MixtureModel();
  virtual ~MixtureModel();
  
  virtual void classify(Classification *classification, Vector *feature);
  
  virtual void scaleDets();
  virtual void show();  
};

#endif

