
#ifndef HD_INFLUENCE_H
#define HD_INFLUENCE_H
#include <stdio.h>

typedef struct _hd_influence hd_influence;
typedef struct _hd_sample hd_sample;
typedef struct _hd_sample_view hd_sample_view;

hd_influence * hd_influence_init(int num_chains, 
                                 double * num_states, 
                                 double * num_outputs,
                                 double * o_type);
void hd_influence_free(hd_influence *);
int hd_influence_get_nchains(hd_influence *infl);
int hd_influence_get_num_states(hd_influence *infl, double *n_states);
int hd_influence_get_num_outputs(hd_influence *infl,double *n_outputs);
int hd_influence_get_o_types(hd_influence *infl,double *o_types);

int hd_influence_set_prior(hd_influence * infl, double *prior);
int hd_influence_get_prior(hd_influence * infl, double *prior);
int hd_influence_set_D(hd_influence * infl, double * D);
int hd_influence_get_D(hd_influence * infl, double * D);
int hd_influence_set_A(hd_influence * infl, int c1, int c2, double *A);
int hd_influence_get_A(hd_influence * infl, int c1, int c2, double *A);
int hd_influence_set_H(hd_influence * infl);
int hd_influence_get_H(hd_influence * infl, double *H);
int hd_influence_set_B(hd_influence * infl, int c, double * B);
int hd_influence_get_B(hd_influence * infl, int c, double * B);
int hd_influence_set_cov(hd_influence * infl, int c, int s, double * cov);
int hd_influence_get_cov(hd_influence * infl, int c, int s, double * cov);
int hd_influence_print(FILE * fd, hd_influence *);

int hd_influence_sample(hd_influence *infl, hd_sample *sampl);
int hd_influence_enter_evidence(hd_influence *,hd_sample *);
int hd_influence_find_mpe(hd_influence *, hd_sample *);
int hd_influence_reset_ess(hd_influence *infl);
int hd_influence_maximize_params(hd_influence * infl);
double hd_influence_loglikelihood(hd_influence * infl, hd_sample * ev);

hd_sample * hd_sample_alloc(hd_influence * infl, int T);
void hd_sample_free(hd_sample * sampl);
int hd_sample_get_nchains(hd_sample *sampl);
int hd_sample_get_num_states(hd_sample *sampl, double *n_states);
int hd_sample_get_num_outputs(hd_sample *sampl,double *n_outputs);
int hd_sample_get_o_types(hd_sample *sampl, double * o_types);
int hd_sample_get_s(hd_sample * sampl, int chain, int t, double * s);
int hd_sample_set_s(hd_sample * sampl, int chain, int t, double s);
int hd_sample_get_o(hd_sample * sampl, int chain, int t, double * o);
int hd_sample_set_o(hd_sample * sampl, int chain, int t, double * o);
int hd_sample_get_gamma(hd_sample * sampl, int chain, int t, double * g);

int hd_sample_print(FILE * fd, hd_sample * sampl);
int hd_sample_print_statistics(FILE * fd, hd_sample *sampl);


hd_sample_view *hd_sample_view_alloc(hd_influence *infl, int buffSize, 
				     int windowSize);
void hd_sample_view_free(hd_sample_view *sampl_view);
/*returned hd_sample* can not be freed*/
hd_sample *hd_sample_view_shift(hd_sample_view *sampl_view, int tShift);

#endif
