/*****************************************************************************
*
* Author: Steve Dunn, MIT Wearable Computing Group
* http://www.media.mit.edu/wearables/
* Copyright (C) 2002 MIT Media Lab
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
*****************************************************************************
* Modified by Mark Blum (mark@media.mit.edu) in July 2005
* Modified by Jon Gips (jgips@media.mit.edu) in August 2005
*
* The MixtureModel class is used by GClassifier. It contains several Gaussians 
* Includes XML parsing
*****************************************************************************/


#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <strings.h>
#include <errno.h>
#include <assert.h>

#include "expat.h"

#include "MixtureModel.hpp"
#include "Gaussian.hpp"
#include "Classification.hpp"


#define BUFSIZE 1023

MixtureModel::MixtureModel() {
    gaussians = NULL;
    priors = NULL;
    versionXML = NULL;
    numGaussians = 0;
}

MixtureModel::~MixtureModel() {
    clear();
}


//--------------------------------------------------

void MixtureModel::show() {
    int i;
    printf("MixtureModel::show()\n");
        
    if (priors == NULL || numGaussians==0) {
        printf("empty MixtureModel\n\n");
    }
    else {
        if (priors != NULL){
            printf("Priors:\n");
            priors->show();
            //PrintVector(stdout, priors->v);
        }
        for(i=0; i<numGaussians; i++) {
            gaussians[i].show();
        }
        printf("\n\n");
    }
    fflush(NULL);
}

//--------------------------------------------------

void MixtureModel::scaleDets() {
    double min;
    int i;
    return; /////////////////////////// temporary !!!!!!!!!!!!!!!!!!!
    if(numGaussians == 0)
        return;

    min = gaussians[0].sigma_det;

    for(i=0; i<numGaussians; i++) {
        if(min > gaussians[i].sigma_det)
            min = gaussians[i].sigma_det;
    }

    for(i=0; i<numGaussians; i++) 
        gaussians[i].sigma_det /= min;
}

//--------------------------------------------------

void MixtureModel::classify(Classification *classification, Vector *feature) {
    double prob;
    int index, i;
        //(*feature)*=scalar;

    //PrintVector(stdout,feature);
    classification->zero();
    for(i=0; i<numGaussians; i++)
        classification->addToLabel(gaussians[i].label, gaussians[i].calc(feature)*VectorRef(priors->v,i));

//     for(i=0; i<numGaussians; i++) {
//         index = classification->getIndex(gaussians[i].label);
//         prob = classification->getProbability(index);
//         printf("%g ",prob);
//         printf("%g ",VectorRef(priors->v,i));
//         classification->setProbability(index, prob*VectorRef(priors->v,i));
//     }
//     printf("\n");
    classification->normalize();
}


//--------------------------------------------------------------------


void MixtureModel::clear(void)
{
    if(versionXML) free(versionXML);
    if(priors) delete priors;
    if(gaussians) delete[] gaussians;
    
    versionXML = NULL;
    scalar = -1;
    numGaussians = -1;
    dimension = -1;
    priors = NULL;
    gaussians = NULL;  
}

//--------------------------------------------------------------------

void MixtureModel::startXMLElement(const char *name, const char **atts) 
{
    if(!strcasecmp(name, "mixturemodel")) {
        xml_state = MIXTUREMODEL;
        clear();
        gaussianLoadIdx = 0;
    }
    else if(!strcasecmp(name, "version")) {
        xml_state = VERSION;
    } 
    else if(!strcasecmp(name, "dimension")) {
        xml_state = DIMENSION;
    } 
    else if(!strcasecmp(name, "scalar")) {
        xml_state = SCALAR;
    } 
    else if(!strcasecmp(name, "numclasses")) {
        xml_state = NUMCLASSES;
    } 
    else if(!strcasecmp(name, "priors")) {
        xml_state = PRIORS;
        if(priors == NULL) assert(0);
        pushXMLObject(priors, name, atts);
    } 
    else if(!strcasecmp(name, "gaussian")) {
        xml_state = GAUSSIAN;
        if(gaussians == NULL) assert(0);
        pushXMLObject(&gaussians[gaussianLoadIdx++], name, atts);
    } 
    else {
        printf("Got unknown start tag \"%s\".\n", name);
        assert(0);
    }
}

void MixtureModel::processXMLCharacterData(const char *s, int len)
{
    switch(xml_state) {
        case VERSION:
            versionXML = strdup(s);
            versionXML = MyStrndup(s,(size_t)len);
            break;
        case DIMENSION:  
            dimension = atoi(s);
            break;
        case SCALAR:  
            scalar = atof(s);
            break;
        case NUMCLASSES:  
            numGaussians = atoi(s);
            gaussians = new Gaussian[numGaussians](dimension);
            priors = new XMLVector(numGaussians);
            break;
        default:
            break;
    }
    xml_state = SKIP; //only one parsing necessary per state
} 

void MixtureModel::endXMLElement(const char *name)
{
    if(!strcasecmp(name, "mixturemodel")) {
        if(gaussianLoadIdx != numGaussians) assert(0);
    
        scaleDets();
        popXMLObject();
    }
}
