/* This code originates from http://www2.cs.uregina.ca/~hamilton/courses/831/notes/ml/dtrees/c4.5/tutorial.html */
/* and was adapted to the needs of the MIT Wearable Computing Group by Mark Blum, July 2005 */

/*************************************************************************/
/*									 */
/*		Type definitions for C4.5				 */
/*              -------------------------				 */
/*									 */
/*************************************************************************/

typedef  char	Boolean, *String, *Set;

typedef  int	ItemNo;		/* data item number */
typedef  float	ItemCount;	/* count of (partial) items */

typedef  short	ClassNo,	/* class number, 0..MaxClass */
                DiscrValue;	/* discrete attribute value (0 = ?) */
typedef  short	Attribute;	/* attribute number, 0..MaxAtt */

typedef  union  _attribute_value
{
    DiscrValue	_discr_val;
    float	_cont_val;
}
	 	AttValue, *Description;

#define  CVal(Case,Attribute)   Case[Attribute]._cont_val
#define  DVal(Case,Attribute)   Case[Attribute]._discr_val
#define  Class(Case)		Case[MaxAtt+1]._discr_val

#define  Unknown  -999		/* unknown value for continuous attribute */


#define  BrDiscr	1	/* node types:	branch */
#define  ThreshContin	2	/*		threshold cut */
#define  BrSubset	3	/*		subset test */

                        typedef  struct _tree_record *Tree;
                typedef  struct _tree_record
{
    short	NodeType;	/* 0=leaf 1=branch 2=cut 3=subset */
    ClassNo	Leaf;		/* most frequent class at this node */
    ItemCount	Items,		/* no of items at this node */
                                                        *ClassDist,	/* class distribution of items */
    Errors;		/* no of errors at this node */
    Attribute	Tested; 	/* attribute referenced in test */
    short	Forks;		/* number of branches at this node */
    float	Cut,		/* threshold for continuous attribute */
    Lower,		/* lower limit of soft threshold */
    Upper;		/* upper limit ditto */
    Set         *Subset;	/* subsets of discrete values  */
    Tree	*Branch;	/* Branch[x] = (sub)tree for outcome x */
}
		TreeRec;

#define  IGNORE		1	/* special attribute status: do not use */
#define  DISCRETE	2	/* ditto: collect values as data read */


                        typedef short	RuleNo;			/* rule number */


                typedef struct TestRec *Test;

                struct TestRec
{
    short	NodeType;	/* test type (see tree nodes) */
    Attribute	Tested;		/* attribute tested */
    short	Forks;		/* possible branches */
    float	Cut;		/* threshold (if relevant) */
    Set		*Subset;	/* subset (if relevant) */
};


typedef struct CondRec *Condition;

struct CondRec
{
    Test		CondTest;	/* test part of condition */
    short	TestValue;	/* specified outcome of test */
};


typedef struct ProdRuleRec PR;

struct ProdRuleRec
{
    short	Size;		/* number of conditions */
    Condition	*Lhs;		/* conditions themselves */
    ClassNo	Rhs;		/* class given by rule */
    float	Error,		/* estimated error rate */
    Bits;		/* bits to encode rule */
    ItemNo	Used,		/* times rule used */
    Incorrect;	/* times rule incorrect */
};


typedef struct RuleSetRec RuleSet;

struct RuleSetRec
{
    PR		*SRule;		/* rules */
    RuleNo	SNRules,	/* number of rules */
                                                        *SRuleIndex;	/* ranking of rules */
    ClassNo	SDefaultClass;	/* default class for this ruleset */
};


/*************************************************************************/
/*									 */
/*		Definitions used in C4.5				 */
/*              ------------------------				 */
/*									 */
/*************************************************************************/


#include <stdio.h>
#include <math.h>

#define	 Eof			EOF             /*char read on end of file*/
#define	 Nil			0               /*null pointer*/
#define	 false			0 
#define	 true			1 
#define	 None			-1
#define	 Epsilon                1E-3

long	 random();
#define	 Random			((random()&2147483647) / 2147483648.0)

#define	 Max(a,b)               ((a)>(b) ? a : b) 
#define	 Min(a,b)               ((a)<(b) ? a : b) 
#define	 Round(x)		((int) (x+0.5))
#define	 Log2			0.69314718055994530942
#define	 Log(x)			((x) <= 0 ? 0.0 : log((float)x) / Log2)

#define	 Bit(b)			(1 << (b))
#define	 In(b,s)		((s[(b) >> 3]) & Bit((b) & 07))
#define	 ClearBits(n,s)		memset(s,0,n)
#define	 CopyBits(n,f,t)	memcpy(t,f,n)
#define	 SetBit(b,s)		(s[(b) >> 3] |= Bit((b) & 07))

#define	 ForEach(v,f,l)		for(v=f ; v<=l ; ++v) 

#define	 Verbosity(d)		if(VERBOSITY >= d)

#define	 Check(v,l,h)\
	     if ( v<l||v>h ) {printf("\t** illegal value **\n"); exit(1);}

        
                
/*************************************************************************/
/*									 */
/*		Global data for C4.5					 */
/*		--------------------					 */
/*									 */
/*************************************************************************/


extern  short		MaxAtt,		/* max att number */
			MaxClass,	/* max class number */
			MaxDiscrVal;	/* max discrete values for any att */

extern  ItemNo		MaxItem;	/* max data item number */

extern  Description	*Item;		/* data items */

extern  DiscrValue	*MaxAttVal;	/* number of values for each att */

extern  char		*SpecialStatus;	/* special att treatment */

extern  String		*ClassName,	/* class names */
		  	*AttName,	/* att names */
		  	**AttValName,	/* att value names */
			FileName;	/* family name of files */

extern  Boolean		AllKnown;	/* true if there have been no splits
					   on atts with missing values above
					   the current position in the tree */


/*************************************************************************/
/*									 */
/*		Global parameters for C4.5				 */
/*		--------------------------				 */
/*									 */
/*************************************************************************/


extern  short		VERBOSITY,	/* verbosity level (0 = none) */
			TRIALS;		/* number of trees to be grown */

extern  Boolean		GAINRATIO,	/* true=gain ratio, false=gain */
			SUBSET,		/* true if subset tests allowed */
			BATCH,		/* true if windowing turned off */
			UNSEENS,	/* true if to evaluate on test data */
			PROBTHRESH;	/* true if to use soft thresholds */

extern  ItemNo		MINOBJS,	/* minimum items each side of a cut */
			WINDOW,		/* initial window size */
			INCREMENT;	/* max window increment each iteration */

extern  float		CF;		/* confidence limit for tree pruning */

