/*************************************************
 * File: WavClipExtractor.c
 * Author: Mark Blum (mark@media.mit.edu)
 * Birth Date: 06/15/05
 *
 * Project: ASSIST
 * 
 * 
 * Copyright (C) 2005 MIT Media Lab
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 ************************************************
 * This program will parse a SignalRecorder audio log-file and extract 
 * Wav-clips at the times corresponding to the timestamps of jpg-images
 * store in the directory
 * External components used:]
 * - audio2wave.pl (pearl script)
 * - ascii2bin (binary)
 * - sox (system command)
 ***********************************************/
 
#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#define SAMPLE_RATE 8000

void PrintUsage()
{
    printf("Usage: WavClipExtractor -a {audioFile} -i [imageList.txt] -t [clip length in seconds] -g [amplitude gain] -l [audio vector length] \n");
    printf("       Default options -i [uses all jpg-files in the directory]\n");
    printf("                       -t [20 seconds]\n");
    printf("                       -g [4] (= 12dB)\n");
    printf("                       -l [1024]\n");
}


int main( int argc, char** argv )
{
    int i,k;
    long clipPosition;
    long readAudioTS;
    int seconds = 20;
    int gain = 4;
    char buffer[8192]; //1024 * 8
    char outputFile[31];
    char * imageList = NULL;
    char * audioFile = NULL;
    FILE * imageList_fid;
    FILE * temp_fid;
    FILE * SQL_input_fid;
    FILE * audio_fid;;
    int samplesPerLine = 1024;
    int foundClip = 0;

    if ( argc < 2 )
    {
    //PrintUsage();
    //return -1;
    }
    i = 1;
    while ( i < argc )
    {
        if (argv[i][0] == '-' )
        {
            i++;
            if ( argc == i )
            {
            //usage();
            //return -1;
            }
            switch ( argv[i-1][1] )
            {
                case 'i': {
                    imageList = argv[i];
                    break;
                }
                case 'a': {
                    audioFile = argv[i];
                    break;
                }
                case 't': {
                    seconds = atoi(argv[i]);
                    break;
                }
                case 'g': {
                    gain = atoi(argv[i]);
                    break;
                }
                case 'l': {
                    samplesPerLine = atoi(argv[i]);
                    break;
                }
                default: PrintUsage(); return -1;
            }
            i++;
        }
        else 
        { 
            PrintUsage();
            return -1;
        }
    }

    if (imageList == NULL)
    {
        system("ls *.jpg > imageList.txt");
        imageList = "imageList.txt";
        printf("WavClipExtractor: Created automatic image list 'imageList.txt'\n");
    }
    if (audioFile == NULL)
    {
        PrintUsage();
        return -1;
    }

    printf("WavClipExtractor: image list is '%s', audio file is '%s', clips are %d seconds, gain is %d, audio length is %d\n",imageList, audioFile, seconds, gain, samplesPerLine);
    imageList_fid = fopen(imageList,"r");
    if (imageList_fid == NULL)
    {
        printf("WavClipExtractor: Error: Could not open file %s\n",imageList);
    }
    SQL_input_fid = fopen("SQL_input.txt","w");
    if (SQL_input_fid == NULL)
    {
        printf("WavClipExtractor: Error: Could not create file 'SQL_input.txt'\n");
    }
    audio_fid = fopen(audioFile,"r");
    if (audio_fid == NULL)
    {
        printf("WavClipExtractor: Error: Could not open file %s'\n",audioFile);
    }
    i = 0;
    
    
    
    while ( fgets(buffer, sizeof(buffer), imageList_fid) != NULL)
    {
        if (sscanf(buffer,"%ld.",&clipPosition) != 1)                  
        // (long) clipPosition and (char *) outputFile are basically the same, e.g. 107301521
        {
            printf("WavClipExtractor: Error: sscanf() failed!\n");
            return -1;
        }
        sprintf(outputFile,"%ld",clipPosition);
       
        //this was too slow:
        //sprintf(buffer,"grep %ld -F %s -A %d > %s", clipPosition, audioFile, (int)(seconds*SAMPLE_RATE/samplesPerLine),outputFile);
        //system(buffer);
        
        foundClip = 0;                
        while (fgets(buffer, sizeof(buffer), audio_fid) != NULL) {
            sscanf(buffer,"%ld.",&readAudioTS);
            if (readAudioTS == clipPosition) {
                temp_fid = fopen(outputFile,"w");
                if (temp_fid == NULL)
                {
                    printf("WavClipExtractor: Error, could not create file %s'\n",outputFile);
                }           
                for (k=0; k<(int)(seconds*SAMPLE_RATE/samplesPerLine); k++) {
                    fprintf(temp_fid,buffer);
                    fgets(buffer, sizeof(buffer), audio_fid);
                }
                fclose(temp_fid);
                // rewind the file pointer back to about the previous position in case two clips occur within the time of [seconds] cliplength
                fseek(audio_fid, -seconds*SAMPLE_RATE*5, SEEK_CUR);  
                foundClip = 1;
                break; // we're done with extracting the audio clip from the big file to the small temp-file
            }
        }
        
        if (!foundClip) {
            printf("WavClipExtractor: Timestamp %ld was not found in %s. No clip created! %ld\n",clipPosition,audioFile, readAudioTS);
            fseek(audio_fid, 0, SEEK_SET); //rewind file pointer
            continue;
        }

        sprintf(buffer,"audio2wave.pl %s %d %d",outputFile,gain,samplesPerLine);   // create wav-file
        system(buffer);
        sprintf(buffer,"rm %s",outputFile);              // remove temporary file
        system(buffer);
    
        fprintf(SQL_input_fid,"%s.jpg,%s.wav\n",outputFile,outputFile);   // write entry to an SQL readable input file.

        i++;
        if ( (i%60) == 0) printf("Processed %d clips = %d minutes of audio\n",i,(int)(i/60));
    }
    fclose(audio_fid);
    fclose(imageList_fid);
    fclose(SQL_input_fid);
    //system("ls *.wav > waveList.txt");
    //printf("WavClipExtractor: Created list of wave clips in 'waveList.txt'\n");
    printf("WavClipExtractor: Created komma seperated jpg & wav list in 'SQL_input.txt'\n");
    printf("WavClipExtractor: Finished creating %d clips !\n",i);
    return 0;
}
