/*************************************************
 * File: identity.h
 * Author: Daniel Barkalow
 * Birth Date: 11/21/03
 *
 * Project: Enchantment IPC
 * 
 * Based on Enchantment IPC Whitepaper
 *   by Edward Keyes.
 * 
 * Copyright (C) 2003 MIT Media Lab,
 *               2004 AWare Technologies, Inc.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 *
 *************************************************/

#ifndef ENCHANTMENT_IDENTITY_H
#define ENCHANTMENT_IDENTITY_H

typedef struct Identity Identity;

/* 
 * Reads the identity information from the specified certificate and
 * key files and returns it. 
 */
Identity *NewIdentity(char *cert, char *key);

/* 
 * Reads and uses the identity information from the specified certificate and
 * key files as the default identity. 
 */
int UseIdentity(char *cert, char *key);

/* Returns the default identity if we have a default identity. */
Identity *DefaultIdentity(void);

/* 
 * Returns the default identity or a anonymous identity if there is no default
 * identity.
 */
Identity *FallbackIdentity(void);

/* 
 * Accept the identities given in the specified location as having
 * accurate names. (Note that this only gives limited signing power to
 * these identities, and doesn't authorize endpoints which use them to
 * do anything in particular).
 */
void AcceptIdentities(char *location);

#endif
