/*************************************************
 * File: time-utils.h
 * Birth Date: Aug 23, 2004
 *
 * Project: Enchantment IPC
 * 
 * Based on Enchantment IPC Whitepaper
 *   by Edward Keyes.
 *
 * Copyright (C) 2004 Aware Technologies, Inc.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 * 
 *************************************************/

#ifndef ENCHANTMENT_TIME_UTILS_H
#define ENCHANTMENT_TIME_UTILS_H

#include <sys/time.h>

/* Adds the given number of microseconds to tm. */
void timeval_add(struct timeval *tm, long usecs);

/* Adds the given number of microseconds to tm. */
void timeval_add_many(struct timeval *tm, long long usecs);

/* Returns t1 - t2, in microseconds. */
long timeval_sub(struct timeval *t1, struct timeval *t2);

/* Sets tm to the nearest value which is 0 mod period. */
void timeval_round(struct timeval *tm, unsigned long period);

/* 
 * Returns at as close as possible to the specified time. If actual is
 * provided, it will be set to the actual time that it is returning. 
 */
void wait_until(struct timeval *tm, struct timeval *actual);

/* A regular or mostly-regular sequence of timestamps. */
typedef struct timestamp_sequence timestamp_sequence;

/* 
 * Returns a new timestamp sequence at the given data rate (in Hz),
 * with a timestamp every length samples. If wait is true, the program
 * will wait such that the times when timestamp_sequence_next()
 * returns match the timestamps returned. 
 */
timestamp_sequence *new_timestamp_sequence(float rate, unsigned length,
					   unsigned wait);

/* 
 * Returns the next timestamp in sequence.
 */
struct timeval *timestamp_sequence_next(timestamp_sequence *seq);

#endif
