/*************************************************
 * File: time-axis.h
 * Birth Date: Jan 11, 2004
 *
 * Project: Enchantment IPC
 * 
 * Based on Enchantment IPC Whitepaper
 *   by Edward Keyes.
 *
 * Copyright (C) 2004 MIT Media Lab
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 *************************************************/

#ifndef _TIME_AXIS_H
#define _TIME_AXIS_H

#include <sys/time.h>
#include <X11/Xlib.h>

#include <pthread.h>

typedef struct XViewer 
{
  Display *d;
  Window win;
  unsigned width;
  unsigned height;

  unsigned long timestep;
  struct timeval basetime;

  int visX;
  int clearX;

  pthread_t thread;
  pthread_mutex_t lock;
  pthread_cond_t cond;

  long offset;

  GC *gcs;

  unsigned has_thread : 1;
  unsigned autoclear : 1;
} XViewer;

XViewer *make_XViewer(char *displayname, unsigned root, char *title,
		      unsigned long timestep, unsigned width, unsigned height);

long xpos(XViewer *viewer, struct timeval *sample);

void XViewer_reset(XViewer *viewer, struct timeval *time);

void draw_line(XViewer *viewer, int color, int lastX, int lastY, 
	       struct timeval *time, int y);

void draw_mark(XViewer *viewer, int color, struct timeval *time);

void draw_bar(XViewer *viewer, XImage *im, int lastX, struct timeval *time);

void note_datum(int *above, int *below, int *count, 
		double min, double max, double val);

int rescale(int *above, int *below, int *count, double *min, double *max);

#endif
