/*************************************************
 * File: signals.h
 * Author: Daniel Barkalow
 * Birth Date: 5/18/04
 *
 * Project: Enchantment IPC
 * 
 * Based on Enchantment IPC Whitepaper
 *   by Edward Keyes.
 *
 * Copyright (C) 2004 AWare Inc.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 *
 *************************************************/

#ifndef ENCHANTMENT_PROTOCOLS_H
#define ENCHANTMENT_PROTOCOLS_H

#include "introduction.h"

// ******  Introductory negotiation properties  ******

// *** Properties common throughout protocol ***

// What kind of interaction will this be?

#define PROTOCOL_TYPE "interaction"
#define PROTOCOL_TYPE_WHITEBOARD "whiteboard"
#define PROTOCOL_TYPE_SIGNAL "signal"

extern IntroFormat protocolInteraction;

// Use encryption

#define PROTOCOL_ENCRYPT "encrypt"
#define PROTOCOL_ENCRYPT_SSL "ssl"

extern IntroFormat protocolEncrypt;

// Version

#define PROTOCOL_VERSION "version"
#define PROTOCOL_VERSION_NUMBER 0x000000

extern IntroFormat protocolVersion;

// Unreliable transport

#define PROTOCOL_UNRELIABLE "unreliable"
#define PROTOCOL_UNRELIABLE_UDP "udp"

extern IntroFormat protocolUnreliable;

// *** Properties for clients and servers ***

extern IntroFormat *clientCommonProperties[];
extern IntroFormat *serverCommonProperties[];

// *** Properties for whiteboard clients ***

extern IntroFormat *whiteboardClientProperties[];

// *** Properties for signals ***

#define PROTOCOL_SIGNAL_UNL "unl"

extern IntroFormat signalUNL;

#define PROTOCOL_SIGNAL_TYPE "type"

extern IntroFormat signalType;

#define PROTOCOL_SIGNAL_LENGTH "length"

extern IntroFormat signalLength;

#define PROTOCOL_SIGNAL_LABEL "label"

extern IntroFormat signalLabel;

#define PROTOCOL_SIGNAL_INDEX_LABEL "index-label"

extern IntroFormat signalIndexLabel;

#define PROTOCOL_SIGNAL_SAMPLE_RATE "sample-rate"

extern IntroFormat signalRate;

#define PROTOCOL_SIGNAL_MIN "min"

extern IntroFormat signalMin;

#define PROTOCOL_SIGNAL_MAX "max"

extern IntroFormat signalMax;

extern IntroFormat *generatorProperties[];

extern IntroFormat *receiverProperties[];

#endif
