/*************************************************
 * File: logging.c
 * Birth Date: Dec 03, 2003
 *
 * Project: Enchantment IPC
 * 
 * Based on Enchantment IPC Whitepaper
 *   by Edward Keyes.
 *
 * Copyright (C) 2003 MIT Media Lab
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 *************************************************/

#include <enchantment/types.h>
#include "logging.h"
#include "tree-manip.h"

#include <stdarg.h>
#include <stdlib.h>
#include <string.h>

#ifdef USE_THREADS
#include <pthread.h>
#include <semaphore.h>
#endif

FILE *debug = NULL;

FILE *g_logfile = NULL;

#ifdef USE_THREADS
pthread_mutex_t logLock;
#endif

void plogf(char *format, ...)
{
  va_list args;
  va_start(args, format);
  if (!g_logfile)
    return;
#ifdef USE_THREADS
  pthread_mutex_lock(&logLock);
#endif
  vfprintf(g_logfile, format, args);
  fflush(g_logfile);
#ifdef USE_THREADS
  pthread_mutex_unlock(&logLock);
#endif
}

static int logTreeTest(NodeType *node)
{
  if (!InTree(node))
    {
      fprintf(g_logfile, "!!");
      return 1;
    }
  if (SchemaOf(node)->hardNode)
    fprintf(g_logfile, "h");
  else
    fprintf(g_logfile, " ");
  if (SchemaOf(node)->subscribers && 
      ArrayLength(SchemaOf(node)->subscribers))
    fprintf(g_logfile, "s");
  else
    fprintf(g_logfile, " ");
  return 1;
}

void logNode(NodeType *node)
{
  if (!g_logfile)
    return;
#ifdef USE_THREADS
  pthread_mutex_lock(&logLock);
#endif
  PrintNodeFilter(node, g_logfile, logTreeTest);
  fflush(g_logfile);
#ifdef USE_THREADS
  pthread_mutex_unlock(&logLock);
#endif
}

void logTree()
{
  plogf("Tree:\n");
  logNode(g_globalNode);
}

int OpenLogFile(char *filename)
{
  char *logname;
  if (filename)
    {
      logname = malloc(strlen(filename) + strlen(LOG_FILE) + 2);
      strcpy(logname, filename);
      strcpy(logname + strlen(filename), "/" LOG_FILE);
    }
  else
    {
      if (!DEFAULT_LOG)
	return 0;
      logname = DEFAULT_LOG_FILE;
    }
  
  g_logfile = fopen(logname, "w");
#ifdef USE_THREADS
  pthread_mutex_init(&logLock, NULL);
#endif

  if (filename)
    free(logname);
  return 1;
}

void closeLog()
{
  if (g_logfile)
    fclose(g_logfile);
}
