/*************************************************
 * File: time-axis.c
 * Birth Date: Jan 11, 2004
 *
 * Project: Enchantment IPC
 * 
 * Based on Enchantment IPC Whitepaper
 *   by Edward Keyes.
 *
 * Copyright (C) 2004 MIT Media Lab,
 *               2004 AWare Technologies, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 *************************************************/

#include "XViewer.h"

#include <stdio.h>

// Modifies base such that it is 0 mod period
void syncTime(struct timeval *base, unsigned long period)
{
  base->tv_sec -= base->tv_sec % period;
  base->tv_usec = 0;
  //printf("Sync to %d.%06d with %ld\n", base->tv_sec, base->tv_usec, period);
}

long xpos(struct timeval *base, struct timeval *sample, unsigned long timestep)
{
  long diffms = (sample->tv_usec - base->tv_usec) / 1000 +
    (sample->tv_sec - base->tv_sec) * 1000;
  return (diffms / timestep);
}

void clear(Display *d, Window win, int lastX, int x, int width, int height)
{
  if (lastX > x)
    ;
  else if ((lastX % width) <= (x % width))
    XClearArea(d, win, (lastX + 1) % width, 0, (x - lastX + 5),
	       height, 0);
  else
    {
      XClearArea(d, win, (lastX) % width + 1, 0, width, height, 0);
      XClearArea(d, win, 0, 0, (x - lastX + 5), height, 0);
    }
}

void note_datum(int *above, int *below, int *count, 
		double min, double max, double val)
{
  if (val < min)
    below[0]++;
  else if (val * 3 < min + min + max)
    below[1]++;
  else if (val * 3 < min + max + max)
    {
      below[1]++;
      above[1]++;
    }
  else if (val < max)
    above[1]++;
  else
    above[0]++;
  (*count)++;
}

int rescale(int *above, int *below, int *count, double *min, double *max)
{
  int ret = 0;
  int i;
  int maxd = 0;
  int mind = 0;
  if (above[0] * 10 > *count)
    maxd = 1;
    //    nmax = *max + (*max - *min) / 2;
  else if ((above[0] + above[1]) * 10 < *count)
    maxd = -1;
    //nmax = (*max + *min) / 2;

  if (below[0] * 10 > *count)
    mind = -1;
  //nmin = *min + (*min - *max) / 2;
  else if ((below[0] + below[1]) * 10 < *count)
    mind = 1;
  //nmin = (*max + *min) / 2;

  if (mind || maxd)
    {
      if (1)
	{
	  printf("%d%% above max, %d%% not above threshold\n",
		 above[0] * 100 / *count,
		 100 - (above[0] + above[1]) * 100 / *count);
	  printf("%d%% below min, %d%% not below threshold\n",
		 below[0] * 100 / *count,
		 100 - (below[0] + below[1]) * 100 / *count);
	}
      if (maxd == mind)
	{
	  // move the window
	  double step = (*max - *min) / 2 * maxd;
	  *min += step;
	  *max += step;
	}
      else if (maxd > 0 && mind < 0)
	{
	  double step = (*max - *min) / 2;
	  // double the window
	  *min -= step;
	  *max += step;
	}
      else if (maxd < 0 && mind > 0)
	{
	  double step = (*max - *min) / 4;
	  // halve the window
	  *min += step;
	  *max -= step;
	}
      else if (maxd > 0)
	{
	  double step = (*max - *min);
	  // double the window
	  *max += step;
	}
      else if (mind < 0)
	{
	  double step = (*max - *min);
	  // double the window
	  *min -= step;
	}
      else if (maxd < 0)
	{
	  double step = (*max - *min) / 2;
	  // halve the window
	  *max -= step;
	}
      else if (mind > 0)
	{
	  double step = (*max - *min) / 2;
	  // halve the window
	  *min += step;
	}
      ret = 1;
    }
  for (i = 0; i < 2; i++)
    {
      //printf("above: %d below: %d\n", above[i], below[i]);
      above[i] = 0;
      below[i] = 0;
    }
  *count = 0;
  return ret;
}
