/*************************************************
 * File: whiteboard-protocol.h
 * Birth Date: Jul 28, 2004
 *
 * Project: Enchantment IPC
 * 
 * Based on Enchantment IPC Whitepaper
 *   by Edward Keyes.
 *
 * Copyright (C) 2002 MIT Media Lab,
 *               2004 Aware Technologies, Inc.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 * 
 *************************************************/

#ifndef ENCHANTMENT_WHITEBOARD_PROTOCOL
#define ENCHANTMENT_WHITEBOARD_PROTOCOL

#define DEFAULT_PORT 11111

#define F_LOWBIT0               (0x1)
#define F_LOWBIT1               (0x2)
#define F_LOWBIT2               (0x4)
#define F_LOWBIT3               (0x8)

#define K_UNL                      "kn" // <K_UNL, "host:/path/nodename">
#define K_AUTH                     "auth"

#define K_PROPERTY_ENCRYPT	    "kpe"
#define K_OPERATION                 "ko" //    
#define K_TYPE			    "kt"

#define  S_ERROR                    "se" //  <S_ERROR, SV_xxxxxx>
#define   SV_BAD_REQUEST             "sbr"
#define   SV_BAD_UNL                 "sbu" 
#define   SV_DOES_NOT_EXIST          "sd" 
#define   SV_FAILURE                 "sf"
#define   SV_SUCCESS                 "ss"
#define   SV_SYMLINK                 "ssl"
#define   SV_ERROR                   "sve"
#define   SV_LOCKED                  "sl"

#define S_RETURN                    "sr" // <S_RETURN>  SYNCHRONOUS

#define S_UPDATE                    "su" // <S_UPDATE>

#define S_TERMINAL                  "st" // <S_TERMINAL> server is going down!


#define C_INITIAL                   "ci" // <C_INITIAL>

#define C_ADMIN                     "cd"

#define C_UNSUBSCRIBE               "cS" // <C_UNSUBSCRIBE, "UNL to unsubscribe to">

#define C_SUBSCRIBE                 "cs" // <C_SUBSCRIBE, "UNL to subscribe to">
#define  F_SUBSCRIBE_SELF             F_LOWBIT0
#define  F_SUBSCRIBE_ONE	      F_LOWBIT3

// Authenticated to have the given certificate
#define CERTIFICATE_AUTH "certificate:"

// Authenticated to have the given network
// (identifies "localhost" and a private network if one is configured)
#define NETWORK_AUTH "network:"

// Authenticated to have the given informational mark
// (These are at the client's discretion)
#define INFO_AUTH "info:"

// Authenticated to have the given client name
// (Chosen by the server, for a single connection)
#define NAME_AUTH "name:"

#define C_PUBLISH                   "cp" // <C_PUBLISH>
#define  F_PUBLISH_REPLACE	      F_LOWBIT0
#define  F_PUBLISH_HARD		      F_LOWBIT1
#define  F_PUBLISH_PRESERVE	      F_LOWBIT2
#define  F_PUBLISH_SYMUNTRUSTED       F_LOWBIT3
#define KV_MERGE                    "kom" 
#define KV_REPLACE                  "kor"
#define KV_ADDONLY                  "koa"

#define C_GETATTR                   "ca"// <C_GETATTR, "UNL from which to get attributes ">

#define   S_ATTR                    "sa"
#define   S_ATTR_PERM		    "sap"
#define   S_ATTR_LOCK               "sal" // <S_ATTR_LOCK, "Lock owner name">
#define    S_ATTR_LOCK_OWNER        "salo"
#define    S_ATTR_LOCK_TYPE         "salt"
#define     KV_ATTR_LOCK_TYPE_NORMAL "normal"
#define     KV_ATTR_LOCK_TYPE_RECURSE "recurse"
#define     KV_ATTR_LOCK_TYPE_REMOVE "remove"

#define    F_SYM_LOCKED              F_LOWBIT0
#define C_GET                       "cg" // <C_GET, "UNL to get from server">
#define C_UNLOCK                    "cL" // Same low bits as C_LOCK
#define C_LOCK                      "cl" // <C_LOCK, "UNL to lock">
#define  F_LOCK_RECURSE               F_LOWBIT0 //Recursive or single level lock
#define  F_LOCK_REMOVE		      F_LOWBIT2 //Remove node when client exits

#define C_SIGNAL_INIT		    "cg"
//	<K_UNL, UNL of signal>

#define K_PROPERTY                  "kp"

#define MAX_NAME_LEN 80

#define   S_SYMLINK_VAL              "sslv"

char *NicknameUNL(char *nickname);

#endif
